class LoadError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class DataNotFoundError(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)




__version__ = '0.0.8'

__title__ = 'prespy'

__summary__ = 'Package for working with the Neurobehavioural Systems Presentation logfiles within python'

__uri__ = 'https://github.com/gjcooper/prespy'

__license__ = 'GPLv2'

__author__ = 'Gavin Cooper'

__email__ = 'gjcooper@gmail.com'



from setuptools import setup, find_packages
import codecs
import os
import subprocess
import sys

base_dir = os.path.abspath(os.path.dirname(__file__))
src_dir = os.path.join(base_dir, 'src')
# When executing the setup.py, we need to be able to import ourselves, this
# means that we need to add the src/ directory to the sys.path.
sys.path.insert(0, src_dir)
about = {}
with codecs.open(os.path.join(src_dir, 'prespy', '__about__.py')) as f:
    exec(f.read(), about)


def genRST():
    pandoc_call = ['pandoc', '--from=markdown', '--to=rst', 'README.md']
    try:
        output = subprocess.run(pandoc_call, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if output.returncode:
            print(output.stderr)
            sys.exit()
        output = output.stdout
    except AttributeError:
        try:
            output = subprocess.check_output(pandoc_call)
        except subprocess.CalledProcessError:
            sys.exit()
    return output.decode()


# get the dependencies and installs
with codecs.open(os.path.join(base_dir, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if x.startswith('git+')]

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__summary__'],
    long_description=genRST(),
    url=about['__uri__'],
    license=about['__license__'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
    ],
    author=about['__author__'],
    author_email=about['__email__'],
    install_requires=install_requires,
    dependency_links=dependency_links,
    packages=find_packages(where='src', exclude=['tests*']),
    package_dir={'': 'src'},
    include_package_data=True,
    keywords=['Presentation', 'NBS', 'logfile'],
    entry_points={
        'console_scripts': ['pres-scla=prespy.__main__:scla_script'],
    }
)



# coding: utf-8
from .logfile import load
from prespy.exceptions import DataNotFoundError
from collections import defaultdict


def mri_timing(logfile, pulsecode=199, events=[]):
    """Load the logfile and extract timing information for events in relation
    to the first pulsecode detected. Each event is either a string to match to
    a code, or a tuple with the column name and the column value to match"""
    logevts = load(logfile).events
    for evt in logevts:
        if evt.code == pulsecode:
            firstpulsetime = evt.time_sec
            break
    else:
        raise DataNotFoundError('Pulse code {} not found in file: {}'.format(pulsecode, logfile))

    tupleevts = [e for e in events if isinstance(e, tuple)]
    categories = defaultdict(list)
    for evt in logevts:
        if evt.code == pulsecode:
            categories['pulses'].append(evt.time_sec - firstpulsetime)
        elif evt.code in events:
            categories[evt.code].append(evt.time_sec - firstpulsetime)
        for te in tupleevts:
            if evt.data[te[0]] == te[1]:
                categories[te[1]].append(evt.time_sec - firstpulsetime)
    return categories


def write_matlab(mfile, timing_data, dec_places=2):
    """write the timing data to a matlab .m file format"""
    with open(mfile, 'w') as writer:
        for category in timing_data:
            formatted = ' '.join(['{0:.{1}f}'.format(secs, dec_places) for secs in timing_data[category]])
            writer.write('{} = [ {} ]\n'.format(category, formatted))
